import pytest

from model.http.info.endpoint.EndpointModel import BasicEndpoint, AbstractEndpoint, UPPER_PORT_LIMIT


def test_should_be_possible_to_create_endpoint_with_url_and_port():
    endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(
        'localhost', 8080)
    assert endpoint.get_url() == 'localhost'
    assert endpoint.get_port() == 8080
    assert endpoint.dump() == "localhost:8080"


def test_should_not_be_possible_to_create_endpoint_with_emtpy_url():
    with pytest.raises(ValueError):
        endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(port=8080)


def test_should_not_be_possible_to_create_endpoint_with_negative_port():
    with pytest.raises(ValueError):
        endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(
            url='localhost', port=-1)


def test_should_not_be_possible_to_create_endpoint_with_port_bigger_than_uppder_limit():
    with pytest.raises(ValueError):
        endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(
            url='localhost', port=UPPER_PORT_LIMIT + 1)



from model.cli.command.FlagModel import AbstractFlag, AbstractFlagValue
from model.http.info.data.DataModel import HttpDataBuilder
from model.http.info.header.HeaderModel import Headers


class HttpHeadersFlagValue(AbstractFlagValue[HttpDataBuilder]):
    def __init__(self, my_query_builder: HttpDataBuilder, flag_value: str):
        super().__init__(my_query_builder, flag_value)

    def match_value(self) -> None:
        self._my_query_builder.add_headers(
            Headers.from_string(self._my_flag_value))


class HttpHeadersFlag(AbstractFlag[HttpDataBuilder]):
    def __init__(self, my_flag_value: AbstractFlagValue[HttpDataBuilder]):
        super().__init__(my_flag_value)

    def get_flag_descritpion(self) -> str:
        return "This flag allows to specify the request headers."




from model.http.info.header.HeaderModel import Headers
from view.item.DictionaryViewerItem import DictionaryViewerItem


class HeadersWrapper(DictionaryViewerItem):
    DEFAULT_CSS = """
    HeadersWrapper {
        margin: 1 0;
        border: none;
    }

    HeadersWrapper ScrollableContainer {
        border: round #30363d;
        background: #161b22;
        padding: 1 2;
        height: 16;
        scrollbar-color: #30363d;
        scrollbar-background: #0d1117;
        scrollbar-size-vertical: 1;
    }
    """

    def __init__(self, values: Headers = Headers.empty(), title: str = 'Headers', **kwargs):
        super().__init__(values.dump(), title, **kwargs)



import re
from http import HTTPMethod
from re import Pattern

from model.cli.command.FlagModel import AbstractFlag, AbstractFlagValue
from model.cli.command.http.Contants import HTTP_FLAG_METHOD
from model.http.info.data.DataModel import HttpDataBuilder

# cursurx http -method=get


class HttpMethodFlagValue(AbstractFlagValue[HttpDataBuilder]):
    _my_query_builder: HttpDataBuilder
    _my_supported_values: set[str] = {
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.PUT,
        HTTPMethod.DELETE,
        HTTPMethod.PATCH
    }

    def __init__(self, my_query_builder: HttpDataBuilder,
                 flag_value: str):
        super().__init__(my_query_builder, flag_value.upper())
        self.my_query_builder = my_query_builder

    def match_value(self) -> None:
        # match a value from the input val
        if self._my_flag_value not in self._my_supported_values:
            raise ValueError(
                f"The input method {self._my_flag_value} is not supported. All the supported values are: {self._my_supported_values}.")
        self.my_query_builder.add_http_method(
            HTTPMethod(value=self._my_flag_value))


class HttpMethodFlag(AbstractFlag[HttpDataBuilder]):
    def __init__(self, my_flag_value: AbstractFlagValue[HttpDataBuilder]):
        self._my_name = HTTP_FLAG_METHOD
        super().__init__(my_flag_value)

    def get_flag_descritpion(self) -> str:
        return "Allows to specify the http method. The current supported methods are:"



from textual.containers import ScrollableContainer
from textual.widgets import Collapsible

from view.item.CustomScrollableContainer import CustomScrollableContainer


class StringViewerItem(Collapsible):
    _my_scrollable_container: ScrollableContainer

    def __init__(self, content: str, title: str, **kwargs):
        super().__init__(
            CustomScrollableContainer(content),
            title=title,
            **kwargs
        )



import re
from re import Match, Pattern

from model.cli.command.FlagModel import AbstractFlag, AbstractFlagValue
from model.cli.command.http.Contants import HTTP_FLAG_ENDPOINT
from model.http.info.data.DataModel import HttpDataBuilder
from model.http.info.endpoint.EndpointModel import BasicEndpoint

IPV4 = r'(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)'
IPV6 = r'(?:[0-9a-fA-F]{1,4}:){7}[0-9a-fA-F]{1,4}'
IPV6_COMPRESSED = r'(?:[0-9a-fA-F]{0,4}:){2,7}[0-9a-fA-F]{0,4}'
HOST = rf'(?:{IPV4}|{IPV6}|\[{IPV6_COMPRESSED}\]|localhost)'
PORT = r'(?:[1-9]\d{0,4}|[1-5]\d{4}|6[0-4]\d{3}|65[0-4]\d{2}|655[0-2]\d|6553[0-5])'


class HttpEndpointFlagValue(AbstractFlagValue[HttpDataBuilder]):
    # ip:port
    # http://ip:port

    _my_pattern_with_protocol: Pattern[str] = re.compile(
        rf'^(https?://)({HOST}):({PORT})$')

    _my_pattern_without_protocol: Pattern[str] = re.compile(
        rf'^(?:https?://)?({HOST}):({PORT})$')

    def match_value(self) -> None:
        matches: Match | None = self._my_pattern_with_protocol.match(
            self._my_flag_value)
        if matches:
            self._my_query_builder.add_endpoint(
                BasicEndpoint.create_endpoint(url=f"{matches.group(1)}{matches.group(2)}", port=int(matches.group(3))))
        else:
            matches: Match | None = self._my_pattern_without_protocol.match(
                self._my_flag_value)
            if matches:
                self._my_query_builder.add_endpoint(
                    BasicEndpoint.create_endpoint(url=matches.group(1),
                                                  port=int(matches.group(2))))
            else:
                raise Exception(
                    f"Invalid http endpoint value: {self._my_flag_value}, you should specify the ip and the port and the protocol.")


class HttpEndpointFlag(AbstractFlag[HttpDataBuilder]):
    def __init__(self, my_flag_value: AbstractFlagValue[HttpDataBuilder]):
        self._my_name = HTTP_FLAG_ENDPOINT
        super().__init__(my_flag_value)

    def get_flag_descritpion(self) -> str:
        return "Allows top specify the http endpoint."



from abc import ABC
from typing import Any

_SENTINEL = object()


class AbstractBody(ABC):
    """
    This class represents the http's body.
    """
    _my_content: str

    def __init__(self, my_content: str = ''):
        self._my_content = my_content

    def get_content(self) -> str:
        return self._my_content


class Body(AbstractBody):
    def __init__(self, _sentinel: object = None, *, content: str):
        if _sentinel is not _SENTINEL:
            raise ValueError(
                'In order to create the body use the factory http.')
        super().__init__(content)

    @classmethod
    def from_string(cls, content: str = '') -> 'Body':
        """
        This factory method creates a Body from the input content string.
        :param content: input content to store
        :return: a new Body object
        """
        return cls(_sentinel=_SENTINEL, content=content)

    @classmethod
    def empty(cls):
        return cls(_sentinel=_SENTINEL, content='')

    @classmethod
    def from_dict(cls, content: dict[Any, Any]) -> 'Body':
        """
        This factory method creates a Body from the input content dictionary.
        :param content: input content dict to store
        :return: a new Body object
        """
        import json
        if len(content) == 0:
            return cls(_sentinel=_SENTINEL, content='')
        return cls(_sentinel=_SENTINEL, content=json.dumps(content))



import json
from abc import ABC
from typing import Collection, Iterable, Set, Mapping

from model.http.info.IterableContentModel import IterableContent

_SENTINEL = object()


class AbstractParameter(ABC):
    """
    This class represents an abstract representation of a parameter.
    A parameter is a tuple of two strings: a key and a value.
    """
    _my_key: str
    _my_value: str

    def __init__(self, key: str, value: str):
        self._my_key = key
        self._my_value = value

    def get_key(self) -> str:
        """
        Retrusn the parameter's key.
        :return: see above.
        """
        return self._my_key

    def get_value(self) -> str:
        """
        Retrun the parameter's value.
        :return: see above.
        """
        return self._my_value

    def __eq__(self, other) -> bool:
        if not isinstance(other, AbstractParameter):
            return False
        return self.get_key() == other.get_key() and self.get_value() == other.get_value()

    def __hash__(self) -> int:
        return hash((self.get_key(), self.get_value()))


class Parameter(AbstractParameter):
    """
    Actual implementation for a parameter.
    """

    def __init__(self, _sentinel: object = None, *, key: str, value: str):
        if _sentinel is None:
            raise TypeError(
                "In order to create this class, use the factory method.")
        super().__init__(key, value)

    @classmethod
    def from_tuple(cls, parameter: tuple[str, str]) -> 'Parameter':
        """
        Factory method that allows to create the Parameter class from a tuple.
        :param parameter: input parameters.
        :return: see above.
        """
        if len(parameter) != 2:
            raise TypeError("The input tuple has to have a size of two.")
        if parameter[0] == '' or parameter[1] == '':
            raise TypeError("The input values has to be different from None.")
        return cls(_sentinel=_SENTINEL, key=parameter[0], value=parameter[1])

    @classmethod
    def from_key_value(cls, key: str = '', value: str = '') -> 'Parameter':
        """
        Factory class that allows to create the Parameter class from a key and a string.
        :param key: input parameter's key.
        :param value: input parameter's value.
        :return: a new Parameter object.
        """
        if key == '' or value == '':
            raise TypeError("The key and the value cannot be empty.")
        return cls(_sentinel=_SENTINEL, key=key, value=value)


class Parameters(IterableContent[Mapping[str, str]]):
    """
    This class represents a collection of parameters.
    """
    _my_parameters: set[Parameter]

    def __init__(self, _sentinel: object = None, *, my_parameters: set[Parameter]):
        if _sentinel is None:
            raise TypeError(
                'In order to create this class, use the factory method.')
        self._my_parameters = my_parameters

    def get_parameters_set(self) -> Set[Parameter]:
        """
        This method returns the collection of all parameters.
        :return: see above.
        """
        return self._my_parameters

    def dump(self) -> Mapping[str, str]:
        """
        This method returns all the parameters of the class.
        :return: see above
        """
        return {parameter.get_key(): parameter.get_value() for parameter in self._my_parameters}

    def __repr__(self) -> str:
        output: str = ""
        for header in self._my_parameters:
            output += f"\t<{header.get_key()}>: <{header.get_value()}>\n"
        return output

    @classmethod
    def from_list(cls, parameters: Collection[Parameter]) -> 'Parameters':
        """
        Factory method that allows to create the Parameters class from a list of parameters.
        :param parameters: input collection of parameters
        :return: a new Parameters object.
        """
        return cls(_sentinel=_SENTINEL, my_parameters={param for param in parameters if param})

    @classmethod
    def from_json_string(cls, content: str) -> 'Parameters':
        """
        Factory method for creating a new Parameters object from a json string.
        :param content: input json string
        :return: a new Parameters object.
        """
        try:
            params: Mapping[str, str] = json.loads(content)
            return cls(_sentinel=_SENTINEL, my_parameters={Parameter.from_key_value(key, value) for key, value in params.items()})
        except json.JSONDecodeError:
            raise ValueError("The input parameter json string is not valid.")

    @classmethod
    def empty(cls) -> 'Parameters':
        """
        Factory method that creates an empty parameters object.
        :return: see above.
        """
        return cls(_sentinel=_SENTINEL, my_parameters=set())

    @classmethod
    def from_tuples(cls, parameters: Collection[tuple[str, str]]) -> 'Parameters':
        """
        Factory method that allows to create the Parameters class from a list of tuples.
        :param parameters: input collection of parameters
        :return: see above.
        """
        return cls(_sentinel=_SENTINEL, my_parameters={Parameter.from_key_value(param[0], param[1]) for param in parameters})



from abc import ABC, abstractmethod

from typing_extensions import TypeVar, Generic


_T = TypeVar("_T")


class AbstractOutput(ABC, Generic[_T]):
    """
    Wrapper for the output.
    """
    _my_output: _T

    def __init__(self, output: _T):
        self._my_output = output

    def get_output(self) -> _T:
        return self._my_output

    @abstractmethod
    def wrap_output(self) -> str:
        """
        Return a string representation of the command output.
        :return: see above.
        """
        pass



import unittest
from abc import ABC
from http import HTTPStatus

from model.http.info.body.BodyModel import AbstractBody
from model.http.info.content.ContentModel import AbstractContent
from model.http.info.endpoint.EndpointModel import AbstractEndpoint
from model.http.info.header.HeaderModel import Headers, AbstractHeader
from model.http.info.parameter.ParameterModel import Parameters, Parameter, AbstractParameter


class BaseContentTestCase(unittest.TestCase):
    __test__ = False
    _my_endpoint: AbstractEndpoint
    _my_headers: Headers
    _my_parameters: Parameters
    _my_body: AbstractBody
    content: AbstractContent

    def __init__(self, content: AbstractContent, method_name: str = "runTest"):
        super().__init__(method_name)
        self.content = content

    def test_headers(self):
        assert all(
            header in self._my_headers.get_headers_set() for header in self.content.get_headers().get_headers_set())

    def test_parameters(self):
        assert all(
            parameter in self._my_parameters.get_parameters_set()for parameter in self.content.get_parameters().get_parameters_set())

    def test_endpoint(self):
        assert self.content.get_endpoint().dump() == self._my_endpoint.dump()

    def test_body(self):
        assert self.content.get_body().get_content() == self._my_body.get_content()



from http import HTTPStatus

from model.http.info.body.BodyModel import AbstractBody
from model.http.info.content.ContentModel import AbstractContent
from model.http.info.cookie.CookieModel import Cookies
from model.http.info.endpoint.EndpointModel import AbstractEndpoint
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters

_SENTINEL = object()


class ResponseContent(AbstractContent):
    """
    This class represents http Response.
    """
    _my_status: HTTPStatus
    _my_cookies: Cookies

    def __init__(self, _sentinel: object = None, *, endpoint: AbstractEndpoint, status_code: HTTPStatus, headers: Headers, parameters: Parameters, cookies: Cookies, body: AbstractBody) -> None:
        if _sentinel is None:
            raise TypeError(
                "In order to create a Response you have to use the factory http.")
        super().__init__(endpoint, headers, parameters, body)
        self._my_status_code = status_code
        self._my_cookies = cookies

    def get_status_code(self) -> HTTPStatus:
        return self._my_status_code

    def get_cookies(self) -> Cookies:
        return self._my_cookies

    @classmethod
    def create_response(cls, endpoint: AbstractEndpoint, status_code: HTTPStatus, headers: Headers, parameters: Parameters, cookies: Cookies, body: AbstractBody) -> 'ResponseContent':
        return cls(_sentinel=_SENTINEL,
                   endpoint=endpoint,
                   status_code=status_code,
                   headers=headers,
                   parameters=parameters,
                   cookies=cookies,
                   body=body)



from typing import Set

import pytest

from model.http.info.header.HeaderModel import Headers, Header, AbstractHeader


def test_should_not_have_duplicate_headers() -> None:
    headers: Headers = Headers.from_list([
        Header.from_tuple(('a', 'b')),
        Header.from_tuple(('a', 'b')),
        Header.from_tuple(('c', 'd'))
    ])
    assert len(headers.get_headers_set()) == 2

    expected: Set[AbstractHeader] = {
        Header.from_tuple(('a', 'b')),
        Header.from_tuple(('c', 'd'))
    }

    assert all(header in expected for header in headers.get_headers_set())


def test_should_raise_error_when_input_collection_is_empty() -> None:
    with pytest.raises(ValueError):
        headers: Headers = Headers.from_list(headers=[])



from http import HTTPStatus

from model.cli.command.OutputModel import AbstractOutput
from model.http.info.content.ResponseModel import ResponseContent


class HttpOutput(AbstractOutput[ResponseContent]):
    def __init__(self, output: ResponseContent):
        super().__init__(output)

    def wrap_output(self) -> str:
        res: ResponseContent = self.get_output()
        status_code: HTTPStatus = res.get_status_code()

        output = "╭" + "─" * 60 + "╮\n"
        output += f"│ ⚡ HTTP RESPONSE: {status_code} ({status_code.name})\n"
        output += self._draw_line()

        output += "│ 📋 HEADERS\n"
        output += f"│ {res.get_headers()}\n"
        output += self._draw_line()

        output += "│ 📋 COOKIES\n"
        output += f"│ {res.get_cookies()}\n"
        output += self._draw_line()

        output += "│ 📄 BODY\n"
        body_content: str = res.get_body().get_content()
        indented_body = "\n".join(
            [f"│ {line}" for line in body_content.splitlines()])
        output += f"{indented_body}\n"

        output += "╰" + "─" * 60 + "╯"
        return output

    def _draw_line(self, length: int = 60) -> str:
        return "├" + "─" * length + "┤\n"



from http import HTTPStatus

import requests
from requests import Response

from model.cli.command.http.Contants import STANDARD_ENGINE_NAME
from model.http.engine.HttpEngineModel import AbstractHttpEngine
from model.http.info.body.BodyModel import Body
from model.http.info.cookie.CookieModel import Cookies
from model.http.info.header.HeaderModel import Headers
from model.http.info.data.DataModel import AbstractHttpData
from model.http.info.parameter.ParameterModel import Parameters
from model.http.info.content.ResponseModel import ResponseContent


class StandardHttpEngine(AbstractHttpEngine):
    def __init__(self):
        super().__init__(STANDARD_ENGINE_NAME)

    def _do_get(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_post(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_put(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_delete(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(), http_request)

    def _do_patch(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(), http_request)

    def _execute_request(self, method: str, request: AbstractHttpData) -> ResponseContent:
        response: Response = requests.request(method=method,
                                              url=request.get_http_request().get_endpoint().dump(),
                                              params=request.get_http_request().get_parameters().dump(),
                                              headers=request.get_http_request().get_headers().dump(),
                                              data=request.get_http_request().get_body().get_content())
        return ResponseContent.create_response(endpoint=request.get_http_request().get_endpoint(),
                                               status_code=HTTPStatus(
                                                   response.status_code),
                                               headers=Headers.from_dictionary(
                                                   response.headers),
                                               parameters=Parameters.from_list(
                                                   []),
                                               cookies=Cookies.from_string_collection(
                                                   response.cookies.items()),
                                               body=Body.from_string(response.text))



from abc import ABC
from http import HTTPMethod

from model.builder.BuilderModel import AbstractBuilder
from model.http.info.body.BodyModel import AbstractBody, Body
from model.http.info.content.RequestModel import RequestContent
from model.http.info.endpoint.EndpointModel import AbstractEndpoint, BasicEndpoint
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters

_SENTINEL = object()


class AbstractHttpData(ABC):
    _my_http_method: HTTPMethod
    _my_http_request: RequestContent

    def __init__(self, method: HTTPMethod, request: RequestContent):
        self._my_http_method = method
        self._my_http_request = request

    def get_http_request(self) -> RequestContent:
        return self._my_http_request

    def get_http_method(self) -> HTTPMethod:
        return self._my_http_method


class HttpData(AbstractHttpData):
    def __init__(self, _sentinel: object = None, *, method: HTTPMethod, request: RequestContent):
        if _sentinel is None:
            raise TypeError(
                "In order to create this class you have to use the factory method.")
        super().__init__(method, request)

    @classmethod
    def create_data(cls, method: HTTPMethod, request: RequestContent):
        if method is None or request is None:
            raise ValueError("The input parameters cannot be None.")
        return cls(_sentinel=_SENTINEL, method=method, request=request)


class HttpDataBuilder(AbstractBuilder[HttpData]):
    _my_endpoint: AbstractEndpoint | None
    _my_headers: Headers = Headers.empty()
    _my_parameters: Parameters = Parameters.empty()
    _my_body: AbstractBody = Body.empty()
    _my_http_method: HTTPMethod | None

    def __init__(self):
        self._my_endpoint: AbstractEndpoint | None = None
        self._my_headers: Headers = Headers.empty()
        self._my_parameters: Parameters = Parameters.empty()
        self._my_body: AbstractBody = Body.empty()
        self._my_http_method: HTTPMethod | None = None

    def add_http_method(self, method: HTTPMethod) -> 'HttpDataBuilder':
        self._my_http_method = method
        return self

    def add_endpoint(self, endpoint: AbstractEndpoint) -> 'HttpDataBuilder':
        self._my_endpoint = endpoint
        return self

    def add_headers(self, headers: Headers) -> 'HttpDataBuilder':
        self._my_headers = headers
        return self

    def add_parameters(self, parameters: Parameters) -> 'HttpDataBuilder':
        self._my_parameters = parameters
        return self

    def add_body(self, body: AbstractBody) -> 'HttpDataBuilder':
        self._my_body = body
        return self

    def build(self) -> HttpData:
        if self._my_http_method and self._my_headers and self._my_parameters and self._my_body and self._my_endpoint:
            return HttpData.create_data(self._my_http_method, RequestContent.create_request(endpoint=self._my_endpoint,
                                                                                            headers=self._my_headers,
                                                                                            parameters=self._my_parameters,
                                                                                            body=self._my_body))
        raise TypeError(
            "In order to create a HttpData class you have to specify all the parameters")



from abc import ABC
from collections.abc import Mapping

from model.http.info.IterableContentModel import IterableContent, _T

_SENTINEL = object()


class AbstractCookie(ABC):
    """
    This class represents the Http Response Cookie.
    """
    _my_cookie_name: str
    _my_cookie_value: str

    def __init__(self, cookie_name: str, cookie_value: str):
        self.my_cookie_name = cookie_name
        self.my_cookie_value = cookie_value

    def get_cookies_name(self) -> str:
        return self.my_cookie_name

    def get_cookies_value(self) -> str:
        return self.my_cookie_value


class Cookie(AbstractCookie):
    """
    Wrapper class for single cookie.
    """

    def __init__(self, _sentinel: object = None, *, cookie_name: str, cookie_value: str):
        if _sentinel is None:
            raise Exception(
                "In order to create the Cookie class you have to use the factory methods.")
        super().__init__(cookie_name, cookie_value)

    @classmethod
    def from_key_value(cls, name: str, value: str) -> 'Cookie':
        if name == "" or value == "":
            raise ValueError("One of the input cookie's parameter are empty.")
        return cls(_sentinel=_SENTINEL, cookie_name=name, cookie_value=value)


class Cookies(IterableContent[Mapping[str, str]]):
    """
    Collection of Cookie objects.
    """
    _my_cookies: Mapping[str, AbstractCookie]

    def __init__(self, _sentinel: object = None, *, cookies: Mapping[str, AbstractCookie]):
        if _sentinel is None:
            raise Exception(
                "In order to create the Cookies class you have to use the factory methods.")
        self._my_cookies = cookies

    def dump(self) -> Mapping[str, str]:
        return {name: value.get_cookies_value() for name, value in self._my_cookies.items()}

    def get_cookie(self, cookie_name: str) -> AbstractCookie:
        return self._my_cookies[cookie_name]

    def __repr__(self):
        output: str = ""
        for cookie, val in self._my_cookies.items():
            output += f"\t<{cookie}>: <{val.get_cookies_value()}>\n"
        return output

    @classmethod
    def from_collection(cls, cookies: list[AbstractCookie]) -> 'Cookies':
        if not cookies:
            raise ValueError("No cookies were given.")
        return cls(_sentinel=_SENTINEL, cookies={cookie.get_cookies_name(): cookie for cookie in cookies})

    @classmethod
    def empty(cls) -> 'Cookies':
        return cls(_sentinel=_SENTINEL, cookies={})

    @classmethod
    def from_string_collection(cls, cookies: list[tuple[str, str]]) -> 'Cookies':
        if not cookies:
            return Cookies.empty()
        return cls(_sentinel=_SENTINEL, cookies={name: Cookie.from_key_value(name=name, value=value) for name, value in cookies})



from abc import ABC
from collections.abc import Collection
from typing import Set, Mapping

from model.http.info.IterableContentModel import IterableContent

import json

_SENTINEL = object()


class AbstractHeader(ABC):
    """
    This class represent a basic header implementation. The header class has a heder name
    and a header value like:
    header-name: <my_header_name>
    """
    _my_header_name: str
    _my_header_value: str

    def __init__(self, my_header_name: str = '', my_header_value: str = ''):
        if len(my_header_name) == 0 or len(my_header_value) == 0:
            raise TypeError(f"The header name cannot be None -> {my_header_name},"
                            f" the header value cannot be None -> {my_header_value}.")
        self._my_header_name = my_header_name
        self._my_header_value = my_header_value

    def get_header_name(self) -> str:
        """
        This http retuns the header name.
        :return: see above
        """
        return self._my_header_name

    def get_header_value(self) -> str:
        """
        This http retuns the header value.
        :return: see above
        """
        return self._my_header_value

    def __repr__(self) -> str:
        return f"""
        header-name: <{self._my_header_name}>
        header-value: <{self._my_header_value}>
        """

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, AbstractHeader):
            return False

        other_object: AbstractHeader = other
        if other_object.get_header_name() != self._my_header_name:
            return False

        if other_object.get_header_value() != self._my_header_value:
            return False

        return True

    def __hash__(self) -> int:
        return hash((self._my_header_name, self._my_header_value))


class Header(AbstractHeader):
    """
    Basic header implementation.
    """

    def __init__(self, _sentinel: object = None, *, my_header_name: str, my_header_value: str):
        if _sentinel is not _SENTINEL:
            raise TypeError(
                "In order to create the Header you have to use the factory methods.")
        super().__init__(my_header_name, my_header_value)

    @classmethod
    def from_tuple(cls, value: tuple[str, str] = ('', '')) -> 'AbstractHeader':
        """
        Factory http for creating a Header object from a tuple of two strings.
        :param value: input tuple of two strings in the format HEADER-NAME: HEADER-VALUE
        :return: a new Header object
        """
        if not value or len(value) != 2:
            raise ValueError('The input value must not be empty and not None.')
        return cls(_sentinel=_SENTINEL, my_header_name=value[0], my_header_value=value[1])

    @classmethod
    def from_key_value(cls, key: str = '', value: str = '') -> 'AbstractHeader':
        if key == '' or value == '':
            raise ValueError('The key and value must not be empty.')
        return cls(_sentinel=_SENTINEL, my_header_name=key, my_header_value=value)


class Headers(IterableContent[Mapping[str, str]]):
    """
    This class represents a collection of headers.
    """

    def __init__(self, _sentinel: object = None, *, headers: Set[AbstractHeader]):
        if _sentinel is not _SENTINEL:
            raise TypeError(
                "In order to create the Headers class you have to use the factory methods.")
        self._my_headers = set()
        for header in headers:
            self._my_headers.add(header)

    def get_headers_set(self) -> Set[AbstractHeader]:
        return self._my_headers

    def dump(self) -> Mapping[str, str]:
        return {header.get_header_name(): header.get_header_value() for header in self._my_headers}

    def __repr__(self) -> str:
        output: str = ""
        for header in self._my_headers:
            output += f"\t<{header.get_header_name()}>: <{header.get_header_value()}>\n"
        return output

    @classmethod
    def from_list(cls, headers: Collection[AbstractHeader]) -> 'Headers':
        """
        Factory http for creating a Headers object from a list of header. This factory http avoid to collect None headers.
        :param headers: input collection of headers
        :return: a new Headers object
        """
        if len(headers) == 0:
            raise ValueError('The input collection must not be empty.')
        return cls(_sentinel=_SENTINEL, headers={header for header in headers if header})

    @classmethod
    def empty(cls) -> 'Headers':
        return cls(_sentinel=_SENTINEL, headers=set())

    @classmethod
    def from_dictionary(cls, headers: Mapping[str, str]) -> 'Headers':
        return cls(_sentinel=_SENTINEL, headers={Header.from_key_value(key, value) for key, value in headers.items()})

    @classmethod
    def from_string(cls, content: str) -> 'Headers':
        try:
            headers: Mapping[str, str] = json.loads(content)
            return cls(_sentinel=_SENTINEL, headers={Header.from_key_value(key, value) for key, value in headers.items()})
        except json.JSONDecodeError:
            raise ValueError('The input content is not a valid json string.')



from model.http.info.cookie.CookieModel import Cookies
from view.item.DictionaryViewerItem import DictionaryViewerItem


class CookiesWrapper(DictionaryViewerItem):
    DEFAULT_CSS = """
        CookiesWrapper {
            margin: 1 0;
            border: none;
        }

        CookiesWrapper ScrollableContainer {
            border: round #30363d;
            background: #161b22;
            padding: 1 2;
            height: 16;
            scrollbar-color: #30363d;
            scrollbar-background: #0d1117;
            scrollbar-size-vertical: 1;
        }
        """

    def __init__(self, cookies: Cookies = Cookies.empty(), title: str = "Cookies", **kwargs):
        super().__init__(values=cookies.dump(), title=title, **kwargs)




from textual import events
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.widgets import Footer

from view.CommandWidget import CommandWidget


class CursusxViewerApp(App):
    BINDINGS = [Binding("q", "quit", "Quit")]
    _my_command_widget: CommandWidget

    def __init__(self, command_app: CommandWidget, **kwargs):
        super().__init__(**kwargs)
        self._my_command_widget = command_app

    async def _on_key(self, event: events.Key) -> None:
        if event.key == "Q":
            self.exit()

    def compose(self) -> ComposeResult:
        yield self._my_command_widget
        yield Footer()



import pytest

from model.http.info.parameter.ParameterModel import Parameter, AbstractParameter


def test_should_raise_type_error_when_value_is_empty() -> None:
    with pytest.raises(TypeError):
        Parameter.from_key_value('akey', '')


def test_should_raise_type_error_when_key_is_empty() -> None:
    with pytest.raises(TypeError):
        Parameter.from_key_value('', 'avalue')


def test_should_be_possible_to_create_parameter_from_tuple() -> None:
    parameter = Parameter.from_tuple(('key', 'value'))
    assert parameter.get_key() == 'key'
    assert parameter.get_value() == 'value'


def test_should_raise_error_when_tuple_has_one_element_empty() -> None:
    with pytest.raises(TypeError):
        Parameter.from_tuple(('a name', ''))
    with pytest.raises(TypeError):
        Parameter.from_tuple(('', 'value'))


def test_should_not_be_possible_to_use_parameter_constructor() -> None:
    with pytest.raises(TypeError):
        Parameter(None, key='a name', value='a value')


def test_should_be_possible_to_create_parameter_from_non_empty_values() -> None:
    try:
        Parameter.from_key_value('key', 'value')
    except ValueError:
        pytest.fail('When the input is not None it should be ok.')


def test_should_be_possible_to_retrieve_all_parameter_attributes() -> None:
    current: AbstractParameter = Parameter.from_key_value('key', 'value')
    assert current.get_key() == 'key'
    assert current.get_value() == 'value'



from http import HTTPStatus

from model.http.info.body.BodyModel import Body, AbstractBody
from model.http.info.content.RequestModel import RequestContent
from model.http.info.endpoint.EndpointModel import AbstractEndpoint, BasicEndpoint
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters, Parameter
from tests.model.http.info.content.BaseContentTest import BaseContentTestCase


class TestRequest(BaseContentTestCase):
    __test__ = True
    _my_endpoint: AbstractEndpoint
    _my_headers: Headers
    _my_parameters: Parameters
    _my_body: AbstractBody

    def __init__(self, method: str = "runTest"):
        self._my_endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(
            'localhost', 8080)
        self._my_headers: Headers = Headers.from_dictionary(
            {'content-type': 'application/json'})
        self._my_parameters: Parameters = Parameters.from_list(
            [Parameter.from_tuple(('key', 'value'))])
        self._my_body: AbstractBody = Body.from_string("random")

        super().__init__(RequestContent.create_request(endpoint=self._my_endpoint,
                                                       headers=self._my_headers,
                                                       parameters=self._my_parameters,
                                                       body=self._my_body
                                                       ), method)



from collections.abc import Mapping

from textual.widgets import Collapsible

from view.item.CustomScrollableContainer import CustomScrollableContainer


class DictionaryViewerItem(Collapsible):

    def __init__(self, values: Mapping[str, str], title: str, **kwargs):
        super().__init__(
            CustomScrollableContainer(values),
            title=title,
            **kwargs
        )



import pytest

from model.http.info.header.HeaderModel import Header, AbstractHeader


def test_should_raise_value_error_when_value_is_empty() -> None:
    with pytest.raises(TypeError):
        Header.from_tuple()


def test_should_not_be_possible_to_use_header_constructor() -> None:
    with pytest.raises(TypeError):
        Header(None, my_header_name='a name', my_header_value='a value')


def test_should_be_possible_to_create_header_from_non_empty_tuple() -> None:
    try:
        Header.from_tuple(('content-type', 'text/html'))
    except ValueError:
        pytest.fail('When the input is a tuple it should be ok.')


def test_should_be_possible_to_retrieve_all_header_attributes() -> None:
    current: AbstractHeader = Header.from_tuple(('content-type', 'text/html'))
    assert current.get_header_name() == 'content-type'
    assert current.get_header_value() == 'text/html'


def test_should_represent_the_correct_value() -> None:
    current: AbstractHeader = Header.from_tuple(('content-type', 'text/html'))
    assert current.__repr__().replace(' ', '').strip() == """
    header-name: <content-type>
    header-value: <text/html>
    """.strip().replace(' ', '')



from abc import abstractmethod, ABC
from typing import TypeVar, Generic

from model.cli.command.CommandModel import AbstractCommand
from model.cli.command.http.Contants import HTTP_COMMAND_NAME, STANDARD_ENGINE_NAME
from model.cli.command.http.HttpCommandModel import HttpCommand
from model.http.engine.Engines import get_engine

T = TypeVar('T')


class AbtractParserStrategy(ABC, Generic[T]):
    """
    Abtract class that defines the strategy for parsing the input query of the user.
    This class allows to translate the input command of a given type into the actual command class.
    """
    @abstractmethod
    def parse_command(self, command: T) -> AbstractCommand:
        """
        Parse the input command of the user and translate it into the actual class representation.
        :param command: input command of the user to parse.
        :return: actual class implementation.
        """
        pass


class CliParserStrategy(AbtractParserStrategy[str]):
    """
    Implementation for a CLI parser strategy, this class allows to work with input string type.
    """

    def parse_command(self, command: str) -> AbstractCommand:
        if command == '':
            raise ValueError('Input command is empty!')
        command_and_flags: list[str] = command.split(' ')
        command_name = command_and_flags[0]

        if command_name == HTTP_COMMAND_NAME:
            return HttpCommand(http_engine=get_engine(STANDARD_ENGINE_NAME))
        raise ValueError('Command is invalid!')


class ParserFactory(Generic[T]):
    """
    Factory class that allows to create an instance of the parser strategy.
    In this way different implementation fo the command listener has its own implementation.

    Current supported implementation:
    1. from command line
    """
    _my_strategy: AbtractParserStrategy[T]

    def __init__(self, command_parser: AbtractParserStrategy[T] = CliParserStrategy()):
        self._my_strategy = command_parser

    def from_command(self, input_command: T) -> AbstractCommand:
        return self._my_strategy.parse_command(input_command)



from abc import ABC

_SENTINEL = object()
UPPER_PORT_LIMIT = 65535


class AbstractEndpoint(ABC):
    _my_url: str
    _my_port: int

    def __init__(self, url: str, port: int):
        self._my_url = url
        self._my_port = port

    def get_url(self) -> str:
        return self._my_url

    def get_port(self) -> int:
        return self._my_port

    def dump(self) -> str:
        return f"{self._my_url}:{self._my_port}"


class BasicEndpoint(AbstractEndpoint):

    def __init__(self, _sentinel: object = None, *,  url: str, port: int):
        if _sentinel is None:
            raise TypeError(
                "In order to create the endpoint you should use the factory method.")
        super().__init__(url, port)

    @classmethod
    def create_endpoint(cls, url: str = '', port: int = -1) -> 'BasicEndpoint':
        if url == '':
            raise ValueError("The url cannot be empty.")
        if port < 0 or port > UPPER_PORT_LIMIT:
            raise ValueError("Port must be between 0 and 65535.")
        return cls(_sentinel=_SENTINEL, url=url, port=port)
